/* WARNING: THIS FILE IS AUTOMATICALLY GENERATED FROM A .SM FILE.
 * Changes made here will most likely be overwritten.
 */

/* 
 * (C) 2001 Clemson University and The University of Chicago 
 *
 * See COPYING in top-level directory.
 */


#include <string.h>
#include <assert.h>

#include "server-config.h"
#include "pvfs2-server.h"
#include "pvfs2-attr.h"
#include "request-scheduler.h"
#include "trove.h"

/* prelude state machine:
 * This is a nested state machine that performs initial setup 
 * steps that are common to many server operations.
 * - post the request to the request scheduler
 * - check permissions
 */

/* TODO: document pre and post conditions (what server_op fields do we
 * require, and which ones do we modify?) 
 * TODO: don't forget in particular to document what we do with the 
 * s_op->attr structure, so other state machines know how to use it
 */

static int prelude_req_sched(PINT_server_op *s_op, job_status_s *js_p);
static int prelude_perm_check(PINT_server_op *s_op, job_status_s *js_p);
static int prelude_getattr_if_needed(PINT_server_op *s_op, job_status_s *js_p);

static int check_mode(PVFS_object_attr* attr, PVFS_uid uid, PVFS_gid gid,
    PVFS_permissions mode);

extern PINT_server_trove_keys_s Trove_Common_Keys[];

extern union PINT_state_array_values ST_req_sched[];
extern union PINT_state_array_values ST_getattr_if_needed[];
extern union PINT_state_array_values ST_perm_check[];

struct PINT_state_machine_s pvfs2_prelude_sm =
{
	ST_req_sched,
	"pvfs2_prelude_sm"
};
static union PINT_state_array_values ST_req_sched[] = {
(union PINT_state_array_values) 0,
(union PINT_state_array_values) prelude_req_sched,
(union PINT_state_array_values) 0,
(union PINT_state_array_values) ST_getattr_if_needed,
(union PINT_state_array_values) -1,
(union PINT_state_array_values) 2
};

static union PINT_state_array_values ST_getattr_if_needed[] = {
(union PINT_state_array_values) 0,
(union PINT_state_array_values) prelude_getattr_if_needed,
(union PINT_state_array_values) -1,
(union PINT_state_array_values) ST_perm_check
};

static union PINT_state_array_values ST_perm_check[] = {
(union PINT_state_array_values) 0,
(union PINT_state_array_values) prelude_perm_check,
(union PINT_state_array_values) -1,
(union PINT_state_array_values) 2
};

# 64 "src/server/prelude.sm"


/* prelude_req_sched()
 *
 * posts a request scheduler job
 */
static int prelude_req_sched(PINT_server_op *s_op, job_status_s *js_p)
{
    int ret = -1;

    gossip_debug(SERVER_DEBUG, 
	"(%p) %s (prelude sm) state: req_sched\n", s_op,
	PINT_map_server_op_to_string(s_op->req->op));

    /* this is a bit of a hack; if we are changing server mode we
     * must preserve the old mode value before calling req scheduler
     */
    if(s_op->op == PVFS_SERV_MGMT_SETPARAM &&
	s_op->req->u.mgmt_setparam.param == PVFS_SERV_PARAM_MODE)
    {
	s_op->resp.u.mgmt_setparam.old_value = PINT_req_sched_get_mode();
    }
    
    ret = job_req_sched_post(s_op->req,
	0,
	s_op,
	0,
	js_p,
	&(s_op->scheduled_id),
	server_job_context);

    return(ret);
}

/* prelude_getattr_if_needed()
 *
 * reads basic attributes of target object, if there is a particular target
 * object for the operation
 */
static int prelude_getattr_if_needed(PINT_server_op *s_op, job_status_s *js_p)
{
    int ret = -1;
    PVFS_handle target_handle = 0;
    PVFS_fs_id target_fs_id = 0;
    job_id_t tmp_id;
    int readonly_flag;

    gossip_debug(SERVER_DEBUG, 
	"(%p) %s (prelude sm) state: getattr_if_needed\n", s_op,
	PINT_map_server_op_to_string(s_op->req->op));

    /* first, run a request scheduler utility function that tells us
     * what handle this request will operate on
     */
    ret = PINT_req_sched_target_handle(
        s_op->req, 0, &target_handle, &target_fs_id, &readonly_flag);
    if (ret < 0)
    {
	js_p->error_code = ret;
	return(1);
    }

    /* if the handle is 0, that indicates that the request does not
     * operate on a specific handle, so there is nothing we can do here
     */
    if (target_handle == 0)
    {
	js_p->error_code = 0;
	return(1);
    }

    /* all other operations fall to this point and read basic attribute
     * information 
     */
    memset(&(s_op->ds_attr), 0, sizeof(PVFS_ds_attributes));
    ret = job_trove_dspace_getattr(
        target_fs_id, target_handle, s_op, &(s_op->ds_attr),
        0, js_p, &tmp_id, server_job_context);

    return(ret);
}

/* prelude_perm_check()
 *
 * this really just marks the spot where we would want to do permission checking,
 * it will be replaced by a couple of states that actually perform this task
 * later
 */
static int prelude_perm_check(PINT_server_op *s_op, job_status_s *js_p)
{
    PVFS_object_attr *obj_attr = NULL;
    PVFS_ds_attributes *ds_attr = NULL;

    gossip_debug(SERVER_DEBUG, 
	"(%p) %s (prelude sm) state: perm_check\n", s_op,
	PINT_map_server_op_to_string(s_op->req->op));

    /*
      first we translate the dspace attributes into a more
      convenient server use-able format.  i.e. a PVFS_object_attr
    */
    ds_attr = &(s_op->ds_attr);
    obj_attr = &(s_op->attr);
    PVFS_ds_attr_to_object_attr(ds_attr, obj_attr);
    s_op->attr.mask = PVFS_ATTR_COMMON_ALL;

    /* the next thing we need to do is interpret the error code from reading
     * the attributes.  Normally it is an error if that step failed, but we 
     * have to look for the special case in which we set attributes on a file
     * that did not have attributes previously.
     */
    /* also, we aren't storing any attributes with datafiles yet */
    /* also, we do both a keyval and a bstream flush on a handle, one of which
     * will fail -- or both, if the bstream hasn't been created yet */
    if((js_p->error_code == -TROVE_ENOENT) && 
	(s_op->req->op == PVFS_SERV_SETATTR ||
	 s_op->req->op == PVFS_SERV_IO ||
         s_op->req->op == PVFS_SERV_GETATTR ||
	 s_op->req->op == PVFS_SERV_REMOVE ||
	 s_op->req->op == PVFS_SERV_FLUSH ||
	 s_op->req->op == PVFS_SERV_TRUNCATE))
    {
	js_p->error_code = 0;
    }

    /* anything else we treat as a real error */
    if(js_p->error_code)
    {
	/* convert to a PVFS error code; we don't have any special semantics
	 * for error codes at this particular spot, so just remove trove mask
	 */
	/* NOTE: the negative signs are necessary because the bit masking
	 * loses the sign, unfortunately
	 */
	js_p->error_code = -PVFS_ERROR_CODE(-js_p->error_code);
	assert(js_p->error_code < 0);
	return(1);
    }
	
    /* decide what kind of permission checking to do based on the 
     * request type:
     */
    switch(s_op->req->op)
    {
	/* things that need read permissions */
	case PVFS_SERV_GETATTR:
            /* a datafile will have a 0 mask value here */
            if (s_op->attr.mask == 0)
            {
                js_p->error_code = 0;
                break;
            }
            else
                /* drop through to check mode */
	case PVFS_SERV_READDIR:
	    /* TODO: figure out something cleaner looking than using 
	     * PVFS_U_READ here to indicate we need read access; its 
	     * misleading
	     */
	    js_p->error_code = check_mode(&(s_op->attr),
		s_op->req->credentials.uid,
		s_op->req->credentials.gid,
		PVFS_U_READ);
	    break;

	/* things that need write permissions */
	case PVFS_SERV_CRDIRENT:
	case PVFS_SERV_RMDIRENT:
	    /* TODO: figure out something cleaner looking than using 
	     * PVFS_U_WRITE here to indicate we need write access; its 
	     * misleading
	     */
	    js_p->error_code = check_mode(&(s_op->attr), 
		s_op->req->credentials.uid,
		s_op->req->credentials.gid,
		PVFS_U_WRITE);
	    break;
	/* special case: we only check permissions on setattr if it 
	 * already has attributes
	 */
	case PVFS_SERV_SETATTR:
	    if(s_op->attr.mask == 0)
	    {
		js_p->error_code = 0;
	    }
	    else
	    {
		js_p->error_code = check_mode(&(s_op->attr), 
		    s_op->req->credentials.uid,
		    s_op->req->credentials.gid,
		    PVFS_U_WRITE);
	    }
	    break;

	/* things we don't check permissions on */
	case PVFS_SERV_MGMT_SETPARAM:
	case PVFS_SERV_CREATE:
	case PVFS_SERV_REMOVE:
	case PVFS_SERV_IO:
	case PVFS_SERV_LOOKUP_PATH:
	case PVFS_SERV_TRUNCATE:
	case PVFS_SERV_MKDIR:
	case PVFS_SERV_GETCONFIG:
	case PVFS_SERV_FLUSH:
	case PVFS_SERV_MGMT_NOOP:
	case PVFS_SERV_MGMT_PERF_MON:
	case PVFS_SERV_MGMT_EVENT_MON:
	case PVFS_SERV_STATFS:
	case PVFS_SERV_MGMT_ITERATE_HANDLES:
	case PVFS_SERV_MGMT_DSPACE_INFO_LIST:
	    js_p->error_code = 0;
	    break;

	/* things we shouldn't see here */
	case PVFS_SERV_INVALID: 
	case PVFS_SERV_WRITE_COMPLETION:
	case PVFS_SERV_PERF_UPDATE:
	    assert(0);
	    js_p->error_code = -PVFS_EINVAL;
	    break;
    }

    return(1);
}


/* check_mode()
 *
 * checks permissions based on desired mode
 *
 * returns 0 on success, -PVFS_EPERM on failure
 */
static int check_mode(PVFS_object_attr* attr, PVFS_uid uid, PVFS_gid gid,
    PVFS_permissions mode)
{
    uint32_t perm_mask = (PVFS_ATTR_COMMON_UID |
                          PVFS_ATTR_COMMON_GID | PVFS_ATTR_COMMON_PERM);

    /* if we don't have masks for the permission information that we need,
     * then the system is broken 
     */
    assert((attr->mask & perm_mask) == perm_mask);

    /* give root permissions, no matter what */
    if(attr->owner == 0)
	return(0);
    /* uid test */
    if((attr->owner == uid) && (((attr->perms >> 6) & mode) == mode))
	return(0);
    /* gid test */
    if((attr->group == gid) && (((attr->perms >> 3) & mode) == mode))
	return(0);
    /* other test */
    if((attr->perms & mode) == mode)
	return(0);

    /* access denied */
    return(-PVFS_EPERM);
}


/*
 * Local variables:
 *  mode: c
 *  c-indent-level: 4
 *  c-basic-offset: 4
 * End:
 *
 * vim: ft=c ts=8 sts=4 sw=4 noexpandtab
 */

